package envoy.server.processors;

import java.io.IOException;

import envoy.data.Message.MessageStatus;
import envoy.event.MessageStatusChangeEvent;
import envoy.exception.EnvoyException;
import envoy.server.ConnectionManager;
import envoy.server.ObjectProcessor;
import envoy.server.database.PersistenceManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>MessageStatusChangeProcessor.java</strong><br>
 * Created: <strong>10 Jan 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Server Standalone v0.1-alpha
 */
public class MessageStatusChangeProcessor implements ObjectProcessor<MessageStatusChangeEvent> {

	/**
	 * Creates an instance of @link{MessageStatusChangeProcessor}.
	 *
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public MessageStatusChangeProcessor() {}

	@Override
	public Class<MessageStatusChangeEvent> getInputClass() { return MessageStatusChangeEvent.class; }

	@Override
	public void process(MessageStatusChangeEvent input, long socketId, ObjectWriteProxy writeProxy) throws IOException {
		try {
			if (!(input.get() == MessageStatus.READ))// check that no invalid MessageStatuses are sent
				throw new EnvoyException("Message" + input.getId() + "has an invalid status");

			ConnectionManager			conMan	= ConnectionManager.getInstance();
			PersistenceManager			perMan	= PersistenceManager.getPersistenceManager();
			envoy.server.data.Message	msg		= perMan.getMessageById(input.getId());

			msg.setStatus(input.get());
			msg.setReadDate(input.getDate());

			if (conMan.isOnline(msg.getRecipient().getId())) writeProxy.write(conMan.getSocketId(msg.getRecipient().getId()), input);

			perMan.updateMessage(msg);

		} catch (IOException | EnvoyException e) {
			if (e.getClass() == IOException.class)
				System.err.println("Recipient online. Failed to send MessageStatusChangedMessageStatusChangeEvent at message" + input.getId());
			e.printStackTrace();
		}
	}
}
